@@ -3,9 +3,13 @@
 #' @usage Globals(object, ...)
 #'
 #' @param object A named list.
+#'
 #' @param \dots Not used.
 #'
-#' @return An object of class \code{Future}.
+#' @return An object of class \code{Globals}, which is a \emph{named} list
+#' of the value of the globals, where the element names are the names of
+#' the globals.  Attribute \code{where} is a named list of the same length
+#' and with the same names.
 #'
 #' @seealso
 #' The \code{\link{globalsOf}()} function identifies globals
@@ -15,7 +19,7 @@
 #' @export
 Globals <- function(object = list(), ...) {
   if (!is.list(object)) {
-    stop("Argument 'object' is not a list: ", class(object)[1])
+    stopf("Argument 'object' is not a list: %s", class(object)[1])
   }
 
   if (length(object) > 0) {
@@ -45,24 +49,28 @@ Globals <- function(object = list(), ...) {
 #' @export
 as.Globals <- function(x, ...) UseMethod("as.Globals")
 
+#' @export
+as.Globals.default <- function(x, ...) {
+  stopf("Don't know how to coerce a %s to Globals", class(x)[1])
+}
+
 #' @export
 as.Globals.Globals <- function(x, ...) x
 
 #' @export
 as.Globals.list <- function(x, ...) {
-  ## Use the globals environments as the locals?
-  ## (with emptyenv() as the fallback)
-  where <- attr(x, "where", exact = TRUE)
-  if (is.null(where)) {
-    where <- lapply(x, FUN = function(obj) {
-        e <- environment(obj)
-        if (is.null(e)) e <- emptyenv()
-        e
-    })
-    names(where) <- names(x)
-    attr(x, "where") <- where
+  if (length(x) > 0L) {
+    stop_if_not(!is.null(names(x)))
+  
+    ## Use the globals environments as the locals?
+    ## (with emptyenv() as the fallback)
+    where <- attr(x, "where", exact = TRUE)
+    if (is.null(where)) {
+      where <- lapply(x, FUN = environment_of)
+      names(where) <- names(x)
+      attr(x, "where") <- where
+    }
   }
-
   Globals(x, ...)
 }
 
@@ -93,36 +101,95 @@ as.Globals.list <- function(x, ...) {
 }
 
 
-#' @export
-`$<-.Globals` <- function(x, name, value) {
+assign_Globals <- function(x, name, value) {
+  stop_if_not(is.character(name), !is.na(name), nchar(name) > 0L)
   where <- attr(x, "where", exact = TRUE)
-
+  stop_if_not(!is.null(where))
+  
   ## Remove an element?
   if (is.null(value)) {
-    x[[name]] <- NULL
     where[[name]] <- NULL
   } else {
     ## Value must be Globals object of length one
     if (inherits(value, "Globals")) {
       if (length(value) != 1) {
-        stop("Cannot assign Globals object of length different than one: ",
+        stopf("Cannot assign Globals object of length different than one: %s",
              length(value))
       }
-      x[[name]] <- value[[1]]
       where[[name]] <- attr(value, "where", exact = TRUE)[[1]]
+      value <- value[[1]]
     } else {
-      w <- environment(value)
-      if (is.null(w)) w <- emptyenv()
-      x[[name]] <- value
-      where[[name]] <- w
+      where[[name]] <- environment_of(value)
     }
   }
 
   attr(x, "where") <- where
+
+  ## Avoid call this function recursively
+  class <- class(x)
+  class(x) <- NULL
+  x[[name]] <- value
+  class(x) <- class
+
   invisible(x)
 }
 
 
+#' @export
+`[<-.Globals` <- function(x, names, value) {
+  stop_if_not(
+    length(names) == length(value),
+    is.character(names), !anyNA(names), all(nchar(names) > 0)
+  )
+
+  if (inherits(value, "Globals")) {
+    where <- attr(value, "where")
+  } else if (is.list(value)) {
+    where <- lapply(value, FUN = environment_of)
+  } else {
+    stopf("Unsupported class of 'value': %s", class(value)[1])
+  }
+  stop_if_not(length(where) == length(value))
+
+  x_where <- attr(x, "where", exact = TRUE)
+  stop_if_not(!is.null(x_where))
+
+  class <- class(x)
+  class(x) <- NULL
+  attr(x, "where") <- NULL
+  
+  for (kk in seq_along(value)) {
+    name <- names[kk]
+    value_kk <- value[[kk]]
+    if (is.null(value_kk)) {
+      x[name] <- list(NULL)
+    } else {
+      x[[name]] <- value_kk
+    }
+    x_where[[name]] <- where[[kk]]
+  }
+
+  stop_if_not(length(x_where) == length(x))
+
+  attr(x, "where") <- x_where
+  class(x) <- class
+
+  invisible(x)
+}
+
+
+#' @export
+`$<-.Globals` <- function(x, name, value) {
+  x <- assign_Globals(x, name = name, value = value)
+  invisible(x)
+}
+
+#' @export
+`[[<-.Globals` <- function(x, name, value) {
+  x <- assign_Globals(x, name = name, value = value)
+  invisible(x)
+}
+
 
 #' @export
 c.Globals <- function(x, ...) {
@@ -144,27 +211,22 @@ c.Globals <- function(x, ...) {
 
       names <- names(g)
       stop_if_not(!is.null(names))
-      w <- lapply(g, FUN = function(obj) {
-        e <- environment(obj)
-        if (is.null(e)) e <- emptyenv()
-        e
-      })
+      w <- lapply(g, FUN = environment_of)
       names(w) <- names
     } else {
       if (is.null(name)) {
-        stop("Can only append named objects to Globals list: ", sQuote(mode(g)))
+        stopf("Can only append named objects to Globals list: %s", sQuote(mode(g)))
       }
+      e <- environment_of(g)
       g <- structure(list(g), names = name)
-      e <- environment(g)
-      if (is.null(e)) e <- emptyenv()
       w <- structure(list(e), names = name)
     }
     where <- c(where, w)
     x <- c(x, g)
   }
 
-  class(x) <- clazz
   attr(x, "where") <- where
+  class(x) <- clazz
 
   stop_if_not(
     length(where) == length(x),
